$TITLE An Obstacle Problem
$offsymxref offsymlist offuellist offuelxref
$inlinecom /* */

/* %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
   program to calculate the position of a membrane
   pushed up through a (rectangular) hole in a rigid plate;
   in addition, there are rigid obstacle(s) inside the hole
   (perhaps not at the same level as the plate).
   These obstacles can constrain the membrane from above
   or from below.
   The correct position of the membrane (the position
   of minimum energy) is determined by the minimization
   of a quadratic function of the membrane position,
   subject to the constraints imposed by the hole
   and the obstacle.  The MCP below arises from the
   optimality conditions for this QP.  Reference:

	Ciarlet, Philippe G.  The finite element method for 
	elliptic problems.  North-Holland, 1978.

   this gams file should write a data file (memb.dat)
   which can be used to view the computed membrane.
   If you have matlab and the file viewmemb.m,
   you can see the membrane quite easily.
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%% */

$setglobal m 51
$setglobal n 51
$if exist matglobs.gms $include matglobs.gms

sets
I	/ 0 * %m% /,
J	/ 0 * %n% /;

scalars
xlo	/ 0 /,
xhi	/ 1 /,
ylo	/ 0 /,
yhi	/ 1 /,
dx,
dy,
c	/ 1 /;		/* force constant */

dx = (xhi - xlo) / (card(I)-1);
dy = (yhi - ylo) / (card(J)-1);

parameters
lb(I,J),
ub(I,J);

variables
v(I,J);		/* height of membrane */

equations
dv(I,J);

dv(I,J) ..
  (dy/dx) * (2*v(I,J) - v(I+1,J) - v(I-1,J))
+ (dx/dy) * (2*v(I,J) - v(I,J+1) - v(I,J-1))
=g=
c * dx * dy;

model obst	/ dv.v /;

option solprint = off;
option limrow = 0;
option limcol = 0;
option iterlim = 100000;
option reslim = 36000;

/* begin problems from Dembo & Tulowitzki */
lb(I,J) = sin(3.2*(xlo + dx*(ord(I)-1)))
	* sin(3.3*(ylo + dy*(ord(J)-1)));
ub(I,J) = 2000;
/* the position v is fixed at zero on the boundary */
ub(I,J)$(ord(J) eq card(J) OR ord(J) eq 1) = 0;
ub(I,J)$(ord(I) eq card(I) OR ord(I) eq 1) = 0;
lb(I,J)$(ord(J) eq card(J) OR ord(J) eq 1) = 0;
lb(I,J)$(ord(I) eq card(I) OR ord(I) eq 1) = 0;
v.lo(I,J) = lb(I,J);
v.up(I,J) = ub(I,J);

v.l(I,J) = 1;

$if exist matdata.gms $include matdata.gms

solve obst using mcp;

$libinclude matout v.l I J
$libinclude matout lb I J
$libinclude matout ub I J
