$title   Optimal tax with two factors of production

$if exist matglobs.gms $include matglobs.gms

set	I	commodities			/beer, pizza/;

$if not setglobal getsets $goto processmod
$libinclude matout I
$exit

$label processmod

scalar	lbar	initial labor endowment		/ 2 /
	kbar	initial capital endowment	/ 1 /
	c0	utility index			/ 3 /
	betal	leisure consumption		/ 1 /
	rev	revenues generated		/ 0 /
	sigma	elasticity of substitution in consumption / 0.8 /;


parameter 
*.						For fun, try changing the alphas to 0.8 and 0.2
	alpha(i) labor intensity in production of Y	   / beer 0.8, pizza 0.2 /
	phi(i)	 productivity parameter			   / beer 1, pizza 1 /
	beta(i)	 consumption shares			   / beer 1, pizza 1 / 

VARIABLE

Y(I)	Consumption Commodities
C	Utility Production
G	Government Good Production
P(I)	Price Of Consumption Commodities
PC	Price Of Utility
PL	Price Of Labor
PK	Price Of Capital
PG	Price Of Government Good
HH	Representative Household
GOVT	Government Agent
T(I)	Endogenous Tax Rate
tau(i)	Tax Variables
MU	Shadow Value On Government Constraint;


Y.L(I)	= 1;
C.L	= 1;
P.L(I)	= 1;
PC.L	= 1;
PK.L$kbar = 1;
PG.L	= 1;
MU.L    = 0;
PL.FX	= 1;
T.L(I)  = 0;
G.L	= 0;
T.L(I)	= 0;
GOVT.L	= 0;

*	Delete the comment to check code for kbar = 0
*. kbar = 0;

*	Command a posititve government revenue

rev = 0.5;

**  ------ MCP formulation ------------- **


*	All variables are define in the MPSGE code

EQUATIONS
	PROFIT(I)	Zero Profit For Activity I
	PROFITC		Zero Profit For Welfare Activity
	PROFITG		Zero Profit For Welfare Government Good
	MARKETG		Market Clearance For Govt Good
	MARKET(I)	Market Clearance For Commodity I
	MARKETL		Market Clearance For Labor
	MARKETK		Market Clearance For Capital
	REVENUE		Government Income Balance
	INCOME		Household Income Balance
	REV_CONST	Revenue Constraint
	TAX(I)		Tax Definition  ;


PROFIT(I)..   PL**alpha(I) * PK$kbar**(1-alpha(I))  =E=  phi(I) * P(I);  

PROFITC..     (betal / c0 * PL**(1-sigma) + sum(I, beta(I) / c0 * (P(I)*(1+T(I)))**(1-sigma)) )**(1/(1-sigma)) 
				=E= PC;

PROFITG..     PG =E= PL;

MARKET(I)..   Y(I) * phi(I) =E= (PC/(P(I)*(1+T(I))))**sigma * beta(I) * C;

MARKETL..     PL * lbar =E= GOVT + sum(I, Y(I) * P(I) * phi(I) * alpha(I) ) 
			    + PL * (PC/PL)**sigma * betal * C;

MARKETK$kbar.. PK * kbar =E= sum(I, Y(I) * P(I) * phi(I) * (1 - alpha(I)) ) ;

MARKETG..     G * PG =E= GOVT;

REVENUE..     GOVT =E= sum(I, Y(I) * phi(I) * P(I) * T(I) );

INCOME..      PC * C * c0 =E= PL * lbar + PK$kbar * KBAR;

REV_CONST..   GOVT =E= PL * rev;

TAX(I)..      T(I) =E= MU * tau(I);

variable obj;
equation objdef;

objdef..	obj =e= c;

model optmcp	/PROFIT.Y, PROFITC.C, PROFITG.G, MARKET.P, MARKETL.PL, MARKETK.PK, 
		 MARKETG.PG, REVENUE.GOVT, INCOME.PC, REV_CONST.MU, TAX.T /;

parameter tauvals(I) /
beer 0.5
pizza 0.5
/;

$if exist matdata.gms $include matdata.gms

TAU.fx(I) = tauvals(I);

solve optmcp using mcp;

$libinclude matout C.L
